@@ -1310,7 +1310,9 @@ struct engine_ops _gpgme_engine_ops_gpgconf =
     NULL,		/* verify */
     NULL,		/* getauditlog */
     NULL,               /* setexpire */
+    NULL,               /* setownertrust */
     NULL,               /* opassuan_transact */
+    NULL,               /* getdirect */
     gpgconf_conf_load,
     gpgconf_conf_save,
     gpgconf_conf_dir,
